@@ -41,6 +41,11 @@ import java.awt.*;
 public class SVGReadParam extends ImageReadParam {
     private Paint background;
     private String baseURI;
+    private boolean allowExternalResources = SVGImageReader.DEFAULT_ALLOW_EXTERNAL_RESOURCES;
+
+    public SVGReadParam() {
+        super();
+    }
 
     public Paint getBackgroundColor() {
         return background;
@@ -58,6 +63,14 @@ public class SVGReadParam extends ImageReadParam {
         baseURI = pBaseURI;
     }
 
+    public void setAllowExternalResources(boolean allow) {
+        allowExternalResources = allow;
+    }
+
+    public boolean isAllowExternalResources() {
+        return allowExternalResources;
+    }
+
     @Override
     public boolean canSetSourceRenderSize() {
         return true;
