@@ -32,7 +32,8 @@ package com.twelvemonkeys.imageio.plugins.iff;
 
 import java.io.DataInput;
 import java.io.DataOutput;
-import java.io.IOException;
+
+import static com.twelvemonkeys.lang.Validate.isTrue;
 
 /**
  * BODYChunk
@@ -41,15 +42,20 @@ import java.io.IOException;
  * @version $Id: BODYChunk.java,v 1.0 28.feb.2006 01:25:49 haku Exp$
  */
 final class BODYChunk extends IFFChunk {
-    protected BODYChunk(int pChunkLength) {
-        super(IFF.CHUNK_BODY, pChunkLength);
+    final long chunkOffset;
+
+    BODYChunk(int chunkId, int chunkLength, long chunkOffset) {
+        super(isTrue(chunkId == IFF.CHUNK_BODY || chunkId == IFF.CHUNK_DBOD, chunkId, "Illegal body chunk: '%s'"), chunkLength);
+        this.chunkOffset = chunkOffset;
     }
 
-    void readChunk(DataInput pInput) throws IOException {
+    @Override
+    void readChunk(final DataInput input) {
         throw new InternalError("BODY chunk should only be read from IFFImageReader");
     }
 
-    void writeChunk(DataOutput pOutput) throws IOException {
+    @Override
+    void writeChunk(final DataOutput output)  {
         throw new InternalError("BODY chunk should only be written from IFFImageWriter");
     }
 }
