@@ -35,35 +35,42 @@ package com.twelvemonkeys.imageio.plugins.jpeg;
 import com.twelvemonkeys.imageio.metadata.jpeg.JPEG;
 
 import javax.imageio.IIOException;
+import javax.imageio.plugins.jpeg.JPEGQTable;
 import java.io.DataInput;
 import java.io.IOException;
 
 final class QuantizationTable extends Segment {
 
-    private final int precision[] = new int[4]; // Quantization precision 8 or 16
-    private final int[] tq = new int[4]; // 1: this table is presented
+    private static final int[] ZIGZAG = {
+             0,  1,  5,  6, 14, 15, 27, 28,
+             2,  4,  7, 13, 16, 26, 29, 42,
+             3,  8, 12, 17, 25, 30, 41, 43,
+             9, 11, 18, 24, 31, 40, 44, 53,
+            10, 19, 23, 32, 39, 45, 52, 54,
+            20, 22, 33, 38, 46, 51, 55, 60,
+            21, 34, 37, 47, 50, 56, 59, 61,
+            35, 36, 48, 49, 57, 58, 62, 63
+    };
 
-    final int quantTables[][] = new int[4][64]; // Tables
+    private final int[] precision = new int[4]; // Quantization precision 8 or 16
+    private final boolean[] tq = new boolean[4]; // 1: this table is present
+
+    private final int[][] quantTables = new int[4][64]; // Tables
 
     QuantizationTable() {
         super(JPEG.DQT);
-
-        tq[0] = 0;
-        tq[1] = 0;
-        tq[2] = 0;
-        tq[3] = 0;
     }
 
-    // TODO: Get rid of table param, make it a member?
-    void enhanceTables(final int[] table) throws IOException {
+    // TODO: Consider creating a copy for the decoder here, as we need to keep the original values for the metadata
+    void enhanceTables() {
         for (int t = 0; t < 4; t++) {
-            if (tq[t] != 0) {
-                enhanceQuantizationTable(quantTables[t], table);
+            if (tq[t]) {
+                enhanceQuantizationTable(quantTables[t], ZIGZAG);
             }
         }
     }
 
-    private void enhanceQuantizationTable(final int qtab[], final int[] table) {
+    private void enhanceQuantizationTable(final int[] qtab, final int[] table) {
         for (int i = 0; i < 8; i++) {
             qtab[table[          i]] *=  90;
             qtab[table[(4 * 8) + i]] *=  90;
@@ -122,7 +129,7 @@ final class QuantizationTable extends Segment {
                 throw new IIOException("Unexpected JPEG Quantization Table precision: " + table.precision[t]);
             }
 
-            table.tq[t] = 1;
+            table.tq[t] = true;
 
             if (table.precision[t] == 8) {
                 for (int i = 0; i < 64; i++) {
@@ -152,4 +159,28 @@ final class QuantizationTable extends Segment {
 
         return table;
     }
+
+    public boolean isPresent(int tabelId) {
+        return tq[tabelId];
+    }
+
+    int precision(int tableId) {
+        return precision[tableId];
+    }
+
+    int[] qTable(int tabelId) {
+        return quantTables[tabelId];
+    }
+
+    JPEGQTable toNativeTable(int tableId) {
+        // TODO: Should de-zigzag (ie. "natural order") while reading
+        // TODO: ...and make sure the table isn't "enhanced"...
+        int[] qTable = new int[quantTables[tableId].length];
+
+        for (int i = 0; i < qTable.length; i++) {
+            qTable[i] = quantTables[tableId][ZIGZAG[i]];
+        }
+
+        return new JPEGQTable(qTable);
+    }
 }
