@@ -292,7 +292,26 @@ public final class TIFFWriter extends MetadataWriter {
 
     private int getCount(final Entry entry) {
         Object value = entry.getValue();
-        return value instanceof String ? ((String) value).getBytes(StandardCharsets.UTF_8).length + 1 : entry.valueCount();
+
+        if (value instanceof String) {
+            return computeStringLength((String) value);
+        }
+        else if (value instanceof String[]) {
+            return computeStringLength((String[]) value);
+        }
+        else {
+            return entry.valueCount();
+        }
+    }
+
+    private int computeStringLength(String... values) {
+        int sum = 0;
+
+        for (String value : values) {
+            sum += value.getBytes(StandardCharsets.UTF_8).length + 1;
+        }
+
+        return sum;
     }
 
     private void writeValueInline(final Object value, final short type, final ImageOutputStream stream) throws IOException {
@@ -410,7 +429,9 @@ public final class TIFFWriter extends MetadataWriter {
 
                         break;
                     }
-
+                case TIFF.TYPE_ASCII:
+                    writeStrings(stream, (String[]) value);
+                    break;
                 default:
                     throw new IllegalArgumentException("Unsupported TIFF type: " + type);
             }
@@ -423,9 +444,7 @@ public final class TIFFWriter extends MetadataWriter {
                     stream.writeByte(((Number) value).intValue());
                     break;
                 case TIFF.TYPE_ASCII:
-                    byte[] bytes = ((String) value).getBytes(StandardCharsets.UTF_8);
-                    stream.write(bytes);
-                    stream.write(0);
+                    writeStrings(stream, (String) value);
                     break;
                 case TIFF.TYPE_SHORT:
                 case TIFF.TYPE_SSHORT:
@@ -462,6 +481,13 @@ public final class TIFFWriter extends MetadataWriter {
         }
     }
 
+    private void writeStrings(ImageOutputStream stream, String... values) throws IOException {
+        for (String value : values) {
+            stream.write(value.getBytes(StandardCharsets.UTF_8));
+            stream.write(0);
+        }
+    }
+
     private void writeValueAt(final long dataOffset, final Object value, final short type, final ImageOutputStream stream) throws IOException {
         writeOffset(stream, dataOffset);
         long position = stream.getStreamPosition();
