@@ -45,7 +45,7 @@ import javax.imageio.metadata.IIOMetadataFormatImpl;
 import javax.imageio.spi.ImageReaderSpi;
 import javax.imageio.stream.ImageInputStream;
 import java.awt.*;
-import java.awt.color.ColorSpace;
+import java.awt.color.*;
 import java.awt.image.*;
 import java.io.DataInput;
 import java.io.DataInputStream;
@@ -378,11 +378,10 @@ public final class SGIImageReader extends ImageReaderBase {
         imageInput.seek(imageInput.getFlushedPosition());
     }
 
-    @Override public IIOMetadata getImageMetadata(final int imageIndex) throws IOException {
-        checkBounds(imageIndex);
-        readHeader();
-
-        return new SGIMetadata(header);
+    @Override
+    public IIOMetadata getImageMetadata(final int imageIndex) throws IOException {
+        ImageTypeSpecifier rawType = getRawImageType(imageIndex);
+        return new SGIMetadata(rawType, header);
     }
 
     public static void main(String[] args) throws IOException {
