@@ -50,6 +50,7 @@ import static com.twelvemonkeys.imageio.plugins.tiff.HorizontalDifferencingStrea
  * @version $Id: HorizontalDeDifferencingStream.java,v 1.0 11.03.13 14:20 haraldk Exp$
  */
 final class HorizontalDeDifferencingStream extends InputStream {
+    /// TODO: Create shared version with PSD, or see if we can avoid some duplication?
     // See TIFF 6.0 Specification, Section 14: "Differencing Predictor", page 64.
 
     private final int columns;
@@ -96,7 +97,7 @@ final class HorizontalDeDifferencingStream extends InputStream {
         }
     }
 
-    private void decodeRow() throws EOFException {
+    private void decodeRow() {
         // Un-apply horizontal predictor
         byte original;
         int sample = 0;
