@@ -39,18 +39,17 @@ import javax.imageio.metadata.IIOMetadataFormatImpl;
  * @author <a href="mailto:harald.kuhr@gmail.com">Harald Kuhr</a>
  * @author last modified by $Author: harald.kuhr$
  * @version $Id: TIFFMedataFormat.java,v 1.0 17/04/15 harald.kuhr Exp$
+ * @deprecated Use {@link TIFFImageMetadataFormat} instead.
  */
+@Deprecated
 public final class TIFFMedataFormat extends IIOMetadataFormatImpl {
-    // TODO: Fix typo in class name + rename to TIFFImageMetadataFormat
+    // TODO: Delete class in next version
     private static final TIFFMedataFormat INSTANCE = new TIFFMedataFormat();
 
-    // We'll reuse the metadata formats defined for JAI
-    public static final String SUN_NATIVE_IMAGE_METADATA_FORMAT_NAME = "com_sun_media_imageio_plugins_tiff_image_1.0";
+    public static final String SUN_NATIVE_IMAGE_METADATA_FORMAT_NAME = TIFFImageMetadataFormat.SUN_NATIVE_IMAGE_METADATA_FORMAT_NAME;
 
     public TIFFMedataFormat() {
         super(SUN_NATIVE_IMAGE_METADATA_FORMAT_NAME, CHILD_POLICY_SOME);
-
-        // TODO: Implement!
     }
 
     @Override
