@@ -31,28 +31,6 @@
 
 package com.twelvemonkeys.imageio.plugins.webp;
 
-import java.awt.*;
-import java.awt.color.ICC_ColorSpace;
-import java.awt.color.ICC_Profile;
-import java.awt.image.BufferedImage;
-import java.awt.image.ColorConvertOp;
-import java.awt.image.ColorModel;
-import java.awt.image.DataBuffer;
-import java.awt.image.WritableRaster;
-import java.io.IOException;
-import java.nio.ByteOrder;
-import java.nio.charset.StandardCharsets;
-import java.util.ArrayList;
-import java.util.Iterator;
-import java.util.List;
-
-import javax.imageio.IIOException;
-import javax.imageio.ImageReadParam;
-import javax.imageio.ImageReader;
-import javax.imageio.ImageTypeSpecifier;
-import javax.imageio.metadata.IIOMetadata;
-import javax.imageio.spi.ImageReaderSpi;
-
 import com.twelvemonkeys.imageio.ImageReaderBase;
 import com.twelvemonkeys.imageio.color.ColorProfiles;
 import com.twelvemonkeys.imageio.color.ColorSpaces;
@@ -67,6 +45,25 @@ import com.twelvemonkeys.imageio.util.ImageTypeSpecifiers;
 import com.twelvemonkeys.imageio.util.ProgressListenerBase;
 import com.twelvemonkeys.imageio.util.RasterUtils;
 
+import javax.imageio.IIOException;
+import javax.imageio.ImageReadParam;
+import javax.imageio.ImageReader;
+import javax.imageio.ImageTypeSpecifier;
+import javax.imageio.metadata.IIOMetadata;
+import javax.imageio.spi.ImageReaderSpi;
+import java.awt.*;
+import java.awt.color.*;
+import java.awt.image.*;
+import java.io.IOException;
+import java.nio.ByteOrder;
+import java.nio.charset.StandardCharsets;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.List;
+
+import static java.lang.Math.max;
+import static java.lang.Math.min;
+
 /**
  * WebPImageReader
  */
@@ -99,7 +96,9 @@ final class WebPImageReader extends ImageReaderBase {
     public void setInput(Object input, boolean seekForwardOnly, boolean ignoreMetadata) {
         super.setInput(input, seekForwardOnly, ignoreMetadata);
 
-        lsbBitReader = new LSBBitReader(imageInput);
+        if (imageInput != null) {
+            lsbBitReader = new LSBBitReader(imageInput);
+        }
     }
 
     private void readHeader(int imageIndex) throws IOException {
@@ -141,6 +140,16 @@ final class WebPImageReader extends ImageReaderBase {
                 case WebP.CHUNK_ANIM:
                     // TODO: 32 bit bg color (hint!) + 16 bit loop count
                     //  + expose bg color in std image metadata...
+
+/*
+                                        int b = (int) lsbBitReader.readBits(8);
+                                        int g = (int) lsbBitReader.readBits(8);
+                                        int r = (int) lsbBitReader.readBits(8);
+                                        int a = (int) lsbBitReader.readBits(8);
+
+                                        Color bg = new Color(r, g, b, a);
+                                        short loopCount = (short) lsbBitReader.readBits(16);
+*/
                     break;
 
                 case WebP.CHUNK_ANMF:
@@ -153,7 +162,7 @@ final class WebPImageReader extends ImageReaderBase {
                     Rectangle bounds = new Rectangle(x, y, w, h);
 
                     // TODO: Expose duration/flags in image metadata
-                    int duration = (int) imageInput.readBits(24);
+                    int duration = (int) lsbBitReader.readBits(24);
                     int flags = imageInput.readUnsignedByte(); // 6 bit reserved + blend mode + disposal mode
 
                     frames.add(new AnimationFrame(chunkLength, chunkStart, bounds, duration, flags));
@@ -205,7 +214,7 @@ final class WebPImageReader extends ImageReaderBase {
 
         switch (chunk) {
             case WebP.CHUNK_VP8_:
-                //https://tools.ietf.org/html/rfc6386#section-9.1
+                // https://tools.ietf.org/html/rfc6386#section-9.1
                 int frameType = lsbBitReader.readBit(); // 0 = key frame, 1 = inter frame (not used in WebP)
 
                 if (frameType != 0) {
@@ -265,19 +274,19 @@ final class WebPImageReader extends ImageReaderBase {
                 }
 
                 // RsV|I|L|E|X|A|R
-                int reserved = (int) imageInput.readBits(2);
+                int reserved = lsbBitReader.readBit();
                 if (reserved != 0) {
                     // Spec says SHOULD be 0
                     throw new IIOException(String.format("Unexpected 'VP8X' chunk reserved value, expected 0: %d", reserved));
                 }
 
-                header.containsICCP = imageInput.readBit() == 1;
-                header.containsALPH = imageInput.readBit() == 1; // L -> aLpha
-                header.containsEXIF = imageInput.readBit() == 1;
-                header.containsXMP_ = imageInput.readBit() == 1;
-                header.containsANIM = imageInput.readBit() == 1; // A -> Anim
+                header.containsANIM = lsbBitReader.readBit() == 1; // A -> Anim
+                header.containsXMP_ = lsbBitReader.readBit() == 1;
+                header.containsEXIF = lsbBitReader.readBit() == 1;
+                header.containsALPH = lsbBitReader.readBit() == 1; // L -> aLpha
+                header.containsICCP = lsbBitReader.readBit() == 1;
 
-                reserved = (int) imageInput.readBits(25); // 1 + 24 bits reserved
+                reserved = (int) lsbBitReader.readBits(26); // 2 + 24 bits reserved
                 if (reserved != 0) {
                     // Spec says SHOULD be 0
                     throw new IIOException(String.format("Unexpected 'VP8X' chunk reserved value, expected 0: %d", reserved));
@@ -425,8 +434,7 @@ final class WebPImageReader extends ImageReaderBase {
                 if (header.containsANIM) {
                     AnimationFrame frame = frames.get(imageIndex);
                     imageInput.seek(frame.offset + 16);
-                    opaqueAlpha(destination.getAlphaRaster());
-                    readVP8Extended(destination, param, frame.offset + frame.length);
+                    readVP8Extended(destination, param, frame.offset + frame.length, frame.bounds.width, frame.bounds.height);
                 }
                 else {
                     imageInput.seek(header.offset + header.length);
@@ -452,6 +460,10 @@ final class WebPImageReader extends ImageReaderBase {
     }
 
     private void readVP8Extended(BufferedImage destination, ImageReadParam param, long streamEnd) throws IOException {
+        readVP8Extended(destination, param, streamEnd, header.width, header.height);
+    }
+
+    private void readVP8Extended(BufferedImage destination, ImageReadParam param, long streamEnd, final int width, final int height) throws IOException {
         while (imageInput.getStreamPosition() < streamEnd) {
             int nextChunk = imageInput.readInt();
             long chunkLength = imageInput.readUnsignedInt();
@@ -465,45 +477,16 @@ final class WebPImageReader extends ImageReaderBase {
 
             switch (nextChunk) {
                 case WebP.CHUNK_ALPH:
-                    int reserved = (int) imageInput.readBits(2);
-                    if (reserved != 0) {
-                        // Spec says SHOULD be 0
-                        processWarningOccurred(String.format("Unexpected 'ALPH' chunk reserved value, expected 0: %d", reserved));
-                    }
-
-                    int preProcessing = (int) imageInput.readBits(2);
-                    int filtering = (int) imageInput.readBits(2);
-                    int compression = (int) imageInput.readBits(2);
-
-                    if (DEBUG) {
-                        System.out.println("preProcessing: " + preProcessing);
-                        System.out.println("filtering: " + filtering);
-                        System.out.println("compression: " + compression);
-                    }
-
-                    switch (compression) {
-                        case 0:
-                            readUncompressedAlpha(destination.getAlphaRaster());
-                            break;
-                        case 1:
-                            opaqueAlpha(destination.getAlphaRaster()); // TODO: Remove when correctly implemented!
-//                            readVP8Lossless(destination.getAlphaRaster(), param);
-                            break;
-                        default:
-                            processWarningOccurred("Unknown WebP alpha compression: " + compression);
-                            opaqueAlpha(destination.getAlphaRaster());
-                            break;
-                    }
-
+                    readAlpha(destination, param, width, height);
                     break;
 
                 case WebP.CHUNK_VP8_:
                     readVP8(RasterUtils.asByteRaster(destination.getRaster())
-                            .createWritableChild(0, 0, destination.getWidth(), destination.getHeight(), 0, 0, new int[]{ 0, 1, 2}), param);
+                            .createWritableChild(0, 0, destination.getWidth(), destination.getHeight(), 0, 0, new int[] {0, 1, 2}), param);
                     break;
 
                 case WebP.CHUNK_VP8L:
-                    readVP8Lossless(RasterUtils.asByteRaster(destination.getRaster()), param);
+                    readVP8Lossless(RasterUtils.asByteRaster(destination.getRaster()), param, width, height);
                     break;
 
                 case WebP.CHUNK_ANIM:
@@ -527,6 +510,92 @@ final class WebPImageReader extends ImageReaderBase {
         }
     }
 
+    private void readAlpha(BufferedImage destination, ImageReadParam param, final int width, final int height) throws IOException {
+        int compression = (int) lsbBitReader.readBits(2);
+        int filtering = (int) lsbBitReader.readBits(2);
+        int preProcessing = (int) lsbBitReader.readBits(2);
+        int reserved = (int) lsbBitReader.readBits(2);
+
+        if (reserved != 0) {
+            // Spec says SHOULD be 0
+            processWarningOccurred(String.format("Unexpected 'ALPH' chunk reserved value, expected 0: %d", reserved));
+        }
+
+        if (DEBUG) {
+            System.out.println("preProcessing: " + preProcessing);
+            System.out.println("filtering: " + filtering);
+            System.out.println("compression: " + compression);
+        }
+
+        WritableRaster alphaRaster = destination.getAlphaRaster();
+        switch (compression) {
+            case 0:
+                readUncompressedAlpha(alphaRaster);
+                break;
+            case 1:
+                // Simulate header
+                imageInput.seek(imageInput.getStreamPosition() - 5);
+
+                WritableRaster tempRaster = Raster.createInterleavedRaster(DataBuffer.TYPE_BYTE, destination.getWidth(), destination.getHeight(), 4, null);
+                readVP8Lossless(tempRaster, param, width, height);
+
+                // Copy from green (band 1) in temp to alpha in destination
+                alphaRaster.setRect(tempRaster.createChild(0, 0, tempRaster.getWidth(), tempRaster.getHeight(), 0, 0, new int[] {1}));
+                break;
+            default:
+                processWarningOccurred("Unknown WebP alpha compression: " + compression);
+                opaqueAlpha(alphaRaster);
+                break;
+        }
+
+        if (filtering != AlphaFiltering.NONE) {
+            for (int y = 0; y < destination.getHeight(); y++) {
+                for (int x = 0; x < destination.getWidth(); x++) {
+                    int predictorAlpha = getPredictorAlpha(alphaRaster, filtering, y, x);
+                    alphaRaster.setSample(x, y, 0, alphaRaster.getSample(x, y, 0) + predictorAlpha % 256);
+                }
+            }
+        }
+    }
+
+    private int getPredictorAlpha(WritableRaster alphaRaster, int filtering, int y, int x) {
+        switch (filtering) {
+            case AlphaFiltering.NONE:
+                return 0;
+            case AlphaFiltering.HORIZONTAL:
+                if (x == 0) {
+                    return y == 0 ? 0 : alphaRaster.getSample(0, y - 1, 0);
+                }
+                else {
+                    return alphaRaster.getSample(x - 1, y, 0);
+                }
+            case AlphaFiltering.VERTICAL:
+                if (y == 0) {
+                    return x == 0 ? 0 : alphaRaster.getSample(x - 1, 0, 0);
+                }
+                else {
+                    return alphaRaster.getSample(x, y - 1, 0);
+                }
+            case AlphaFiltering.GRADIENT:
+                if (x == 0) {
+                    return y == 0 ? 0 : alphaRaster.getSample(0, y - 1, 0);
+                }
+                else if (y == 0) {
+                    return alphaRaster.getSample(x - 1, 0, 0);
+                }
+                else {
+                    int left = alphaRaster.getSample(x - 1, y, 0);
+                    int top = alphaRaster.getSample(x, y - 1, 0);
+                    int topLeft = alphaRaster.getSample(x - 1, y - 1, 0);
+
+                    return max(0, min(left + top - topLeft, 255));
+                }
+            default:
+                processWarningOccurred("Unknown WebP alpha filtering: " + filtering);
+                return 0;
+        }
+    }
+
     private void applyICCProfileIfNeeded(final BufferedImage destination) {
         if (iccProfile != null) {
             ColorModel colorModel = destination.getColorModel();
@@ -558,6 +627,7 @@ final class WebPImageReader extends ImageReaderBase {
         }
     }
 
+    @SuppressWarnings("RedundantThrows")
     private void readUncompressedAlpha(final WritableRaster alphaRaster) throws IOException {
         // Hardly used in practice, need to find a sample file
         processWarningOccurred("Uncompressed WebP alpha not implemented");
@@ -565,8 +635,13 @@ final class WebPImageReader extends ImageReaderBase {
     }
 
     private void readVP8Lossless(final WritableRaster raster, final ImageReadParam param) throws IOException {
+        readVP8Lossless(raster, param, header.width, header.height);
+    }
+
+    private void readVP8Lossless(final WritableRaster raster, final ImageReadParam param,
+                                 final int width, final int height) throws IOException {
         VP8LDecoder decoder = new VP8LDecoder(imageInput, DEBUG);
-        decoder.readVP8Lossless(raster, true);
+        decoder.readVP8Lossless(raster, true, param, width, height);
     }
 
     private void readVP8(final WritableRaster raster, final ImageReadParam param) throws IOException {
@@ -588,10 +663,7 @@ final class WebPImageReader extends ImageReaderBase {
 
     @Override
     public IIOMetadata getImageMetadata(int imageIndex) throws IOException {
-        readHeader(imageIndex);
-        readMeta();
-
-        return new WebPImageMetadata(header);
+        return new WebPImageMetadata(getRawImageType(imageIndex), header);
     }
 
     private void readMeta() throws IOException {
